// Global variables
var imagePattern = /^.*\.(gif|jpe?g|png)$/i;
var urlPattern = /http:\/\/([^.]+\.){2,}.*/i;
var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

// Initialize the app on window load
window.addEventListener("load", outfoxemInit, false);

// Add a listener that will parse a page when it is loaded if the flag is set
window.addEventListener("load", function() { outFoxEmPageLoad.init(); }, false);
var outFoxEmPageLoad = {
	init: function() {
		var appcontent = document.getElementById("appcontent");   // browser
		if(appcontent) {
			appcontent.addEventListener("load", this.onPageLoad, true);
		}
  	},

	onPageLoad: function(aEvent) {
		var doc = aEvent.originalTarget;
		var outFoxEmOn = prefs.getBoolPref("outfoxem.enabled");
		
		// If the preference is set to true and the doc being
		// loaded is not an image, parse the page.
		if (outFoxEmOn && !doc.URL.match(imagePattern)) {
			try {
				parsePage(doc);
			} catch (e) {
				var newPage = '<html><head><title>Unable to OutFoxEm!</title></head>';
				newPage += '<body><h3>Unable to OutFoxEm!</h3>';
				newPage += '<p><strong>Please cut and paste the following information below and <a href="mailto:sprak3000@gmail.com?subject=OutFoxEm Report">email it</a> to the author:</strong></p>';
				newPage += '<p>Page URL:<br />' + doc.URL + '</p><p>Error Message:<br />' + e + '</p>';
				newPage += '</body></html>';

				doc.open();
				doc.write(newPage);
				doc.close();
			}
		}
  	}
}

// Event Listener that determines if the popup menu is for right-clicking on a link.
// If so, display our option.
function outfoxemContext()
{
	var menuitem = document.getElementById("context-outfoxem");
	if (menuitem) {
		menuitem.hidden = gContextMenu.onLink;
	}
}

// Initialize the application; add menu event listener to determine when to show menu option.
function outfoxemInit()
{
	var menu = document.getElementById("contentAreaContextMenu");
  	menu.addEventListener("popupshowing", outfoxemContext, false);

	var on = false;

	// If the preference does not exist, set it to false
	if (prefs.getPrefType("outfoxem.enabled") != prefs.PREF_BOOL) {
		prefs.setBoolPref("outfoxem.enabled", false);
	} else {
		on = prefs.getBoolPref("outfoxem.enabled");
	}

	var button = document.getElementById("outfoxem-button");
	button.checked = on;
}

// Parse out all img tags that are contained by an a tag
// that has its href attribute pointing to an image.
function parsePage(doc)
{
	// header text for new HTML page
	var htmlHeader = '<html><head>';


	// Parse out <base> tags and grab the href value if it has it.
	var baseHref = "";
	var baseTags = doc.getElementsByTagName("base");
	if (baseTags.length > 0) {
		for (i = 0; i < baseTags.length; i++) {
			if (baseTags[i].hasAttribute("href") &&
				baseTags[i].getAttribute("href").match(urlPattern) != null) {

				baseHref = baseTags[i].getAttribute("href");
			}
		}
	}

	// If we did not find any <base> tags or a suitable href attribute,
	// use the document URL up to the final /
	if (baseHref == "") {
		baseHref = doc.URL.substring(0, doc.URL.lastIndexOf("/") + 1);
	}

	htmlHeader += '<base href="' + baseHref + '" /><style type="text/css"><!-- table { margin-left:  auto; margin-right:  auto;} img { border:  none; } // --></style></head><body><div style="text-align:  center;"><table><tr>';

	// Grab all links (<a> tags) from the document.
	var links = doc.getElementsByTagName("a");
	var linksCount = links.length;
	var columnCount = 0;
	var newPage = '';

	// Iterate over the links in a document and parse out those linking directly to an image file.
	for (i = 0; i < linksCount; i++) {
		var currentLink = links[i];

		/**
		  * If the current <a> tag has an href attribute and that attribute value
		  * matches the image pattern, grab the values of the tags and add them
		  * to the new HTML document to display.
		  */
		if (currentLink.hasAttribute("href") &&
			currentLink.getAttribute("href").match(imagePattern) != null) {

			var linkHrefValue = currentLink.getAttribute("href");

			// Grab all <img> tags under the <a> tag and parse them.
			var imgChildren = currentLink.getElementsByTagName("img");
			var imgChildrenCount = imgChildren.length;

			for (j = 0; j < imgChildrenCount; j++) {
				if (imgChildren[j].hasAttribute("src")) {
					newPage += '<td><a href="' + linkHrefValue + '">' +
							'<img src="' + imgChildren[j].getAttribute("src") + '" ';

					if (imgChildren[j].hasAttribute("width")) {
						newPage += 'width="' + imgChildren[j].getAttribute("width") + '" ';
					}

					if (imgChildren[j].hasAttribute("height")) {
						newPage += 'height="' + imgChildren[j].getAttribute("height") + '" ';
					}

					newPage += '/></a></td>';

					// Increment the column count and close out a row if necessary
					if (columnCount < 4) {
						newPage += '<td>&nbsp;</td>';
						columnCount++;
					} else {
						newPage += '</tr><tr><td colspan="5">&nbsp;</td></tr><tr>';
						columnCount = 0;
					}
				}
			}
		}
	}

	if (columnCount < 5) {
		newPage += '</tr>';
	}

	// HTML footer for the page
	var htmlFooter = '</table></div></body></html>';

	doc.open();
	doc.write(htmlHeader + newPage + htmlFooter);
	doc.close();
}

// Main function
function outFoxEm()
{
	var outFoxEmOn = prefs.getBoolPref("outfoxem.enabled");

	if (outFoxEmOn) {
		prefs.setBoolPref("outfoxem.enabled", false);
	} else {
		prefs.setBoolPref("outfoxem.enabled", true);
	}
}
